from setuptools import setup

setup(
    name="spin-distributions",
    version="0.1",
    description="Gaussian & Binomial Distributions",
    packages=["spin-distributions"],
    author='Adedayo Agboola',
    author_email='maxkreativity@gmail.com',
    zip_safe=False,
)
